#!/usr/bin/perl

###
# Use Syntax for Vote.pl: 
# (blank) Creates list of ACTIVE and INACTIVE Polls. two separate categories.
# poll=(number) (required to view a poll)
# vote=(number) corresponds to vote option (required to place vote)
#
#	$polls[$poll_num]{'status'} = active|inactive
#	$polls[$poll_num]{'name'} = the question the poll asks
#	$polls[$poll_num]{'file'} = filename of poll file
#	$results[$poll_num]{$ip} = $value;  (the vote for each IP addr
#	$tally[$poll_name][$option_num] = total of votes for this option #
#	$options[$poll_num] = array, each line is an option
#
#
###

require "lib/environment";
require "lib/utils";
&loadenv(%ENV);

chdir "data/vote/";

&load_data;
&load_template;

print "Content-type: text/html\n\n";
print $templ1;


###
# Determine the Action requested.
###

if (!defined($pairs{'poll'})) {
	# This case is to load a List of Active and Inactive polls.
	&poll_list;
} elsif (defined($pairs{'vote'})) {
	# This case is to vote for the poll aforementioned.
	$poll_num = $pairs{'poll'};
	$vote = $pairs{'vote'};
	load_poll($poll_num);
	vote_poll($poll_num,$vote);
} else {
	# This case is to read the poll, and give voting options if open.
	$poll_num = $pairs{'poll'};
	load_poll($poll_num);
	show_poll($poll_num);
}

print $templ2;
exit;

###
# Subroutines to handle cases:
###

# 1- List Polls.

sub poll_list {
	foreach $poll (@polls) {
		if ($$poll{'status'} eq "active") {
			push(@activelist,$$poll{'num'});
		} elsif ($$poll{'status'} eq "inactive") {
			push(@inactivelist,$$poll{'num'});
		}
	}

	print "Active Polls:\n<ul>\n";
	foreach $poll (@activelist) {
		print "<li><a href=\"vote.pl?poll=$poll\">$polls[$poll]{'name'}</a> by $polls[$poll]{'owner'}\n";
	}
	print "</ul>\nInactive Polls:\n<ul>\n";
	foreach $poll (@inactivelist) {
		print "<li><a href=\"vote.pl?poll=$poll\">$polls[$poll]{'name'}</a> by $polls[$poll]{'owner'}\n";
	}
	print "</ul>\n";

#End sub poll_list.
}


sub show_poll {
my($poll_num) = @_;
	if ((&alreadyvoted($poll_num)) or ($polls[$poll_num]{'status'} eq "inactive")) {
		$CanVote = 0;
	} else {
		$CanVote = 1;
	}

	print "<center><h2>$polls[$poll_num]{'name'}</h2>\nby <b>$polls[$poll_num]{'owner'}</b>\n<br><br>\n";

	if ($CanVote) {
		print "<form action=\"vote.pl\" method=\"GET\">\n<input type=\"hidden\" name=\"poll\" value=\"$poll_num\">\n";
	}

	print "<table>\n";

	for ($option_num = 0;$option_num <= $#{$options[$poll_num]};$option_num++) {
		print "<tr><td>\n\t";

		if ($CanVote) {
			print "<input type=\"radio\" name=\"vote\" value=\"$option_num\">\n\t";
		}

		if (votedforthis($poll_num,$option_num)) {print "<b>";}

		print "&nbsp; ($tally[$poll_num][$option_num] Votes) &nbsp; $options[$poll_num][$option_num]\n</td></tr>";

		if (votedforthis($poll_num,$option_num)) {print "</b>";}
		print "\n";
	# End For
	}

	print "</table>\n\n";

	if ($CanVote) {
		print "<input type=\"submit\" value=\"Vote Already!\">\n</form>\n";
	}
#End show_poll sub.
}


sub vote_poll {
my($poll_num,$vote) = @_;

	if (alreadyvoted($poll_num)) {
		print "<center>Sorry, you already voted!</center>\n";
	} else {
		my $ip = $ENV{'REMOTE_ADDR'};
		my $filedata = `cat $polls[$poll_num]{'file'}`;
		($part1,$part2) = split(/<\/results>/i,$filedata);
		$part1 .= "$ip=$vote\n";
		$part2 = "";
		$filedata = $part1."</results>";
		open(FILE,">$polls[$poll_num]{'file'}") or dienice("Could not open poll file for writing.");
		print FILE $filedata;
		close(FILE);
	}
	load_poll($poll_num);
	show_poll($poll_num);
	return;
}
###
### DEBUG
###

sub debug_me {
	print "List of Polls:\n";
	foreach $poll (@polls) {
		if ($$poll{'status'} eq "active") {
			print "  Active Poll: $$poll{'num'} - \"$$poll{'name'}\" by $$poll{'owner'}\n";
		} elsif ($$poll{'status'} eq "inactive") {
			print "Inactive Poll: $$poll{'num'} - \"$$poll{'name'}\" by $$poll{'owner'}\n";
		}

		&load_poll($$poll{'num'});
		$bla = $$poll{'num'};
		print "   options: ".scalar(&get_options($bla))."\n";

#		foreach $option (&get_options($bla)) {
#			print "     $option\n";
#		}

		@this = get_totals($bla);
		for ($n = 0;$n <= $#this;$n++) {
			print "     $this[$n] Votes for: $options[$bla][$n]\n";
		}
	}
# End Sub debug_me.
}


###
# Subroutines to pull simple data from stored arrays.
###

sub get_options {
my($poll) = @_;
	return @{$options[$poll]};
}

sub get_totals {
my($poll) = @_;
	return @{$tally[$poll]};
}

sub load_template {
	my $templ = `cat ../../templates/vote.html`;
	($templ1,$templ2) = split(/<split>/i,$templ);
	$templ = undef;

}

sub alreadyvoted {
my($poll_num) = @_;
	my $ip = $ENV{'REMOTE_ADDR'};

	if (defined($results[$poll_num]{$ip})) {
		return 1;
	} else {
		return 0;
	}
}

sub votedforthis {
my ($poll_num,$option_num) = @_;
my $ip = $ENV{'REMOTE_ADDR'};
	if (!defined($results[$poll_num]{$ip})) {return 0}
	if ($results[$poll_num]{$ip} == $option_num) {
		return 1;
	} else {
		return 0;
	}
}

###
# Subroutines to load/read info
###


sub load_data {
	open(DATA,"vote.dat") or dienice("Could not open Vote.dat for reading");
	my @vdata = <DATA>;
	close(DATA);

	foreach $line (@vdata) {
		chomp($line);
		if (substr($line,0,1) eq "#") {

		} else {
			@ld = split(/\|/i,$line);
			$polls[$ld[0]] = {
				'num' => $ld[0],
				'status' => $ld[1],
				'name' => $ld[2],
				'file' => $ld[3],
				'owner' => $ld[4]
			}
		}
	}
@ld = ();
@vdata = ();
# End load_data sub.
}

sub load_poll {
my($poll_num) = @_;
	open(POLLFILE,"$polls[$poll_num]{'file'}") or dienice("Could not open poll file $polls[$poll_num]{'file'}.");
	@vdata = <POLLFILE>;
	close(POLLFILE);

	my $bleh = "";
	foreach $line (@vdata) {
		$bleh .= $line;
	}
	my $optiondata = between($bleh,"<options>\n","</options>");
	my @dumpme = split(/\n/i,$optiondata);
	for ($n = 0;$n <= $#dumpme;$n++) {
		$options[$poll_num][$n] = $dumpme[$n];
	}
	@dumpme = ();

	$resultdata = between($bleh,"<results>\n","</results>");
	@resultarr = split(/\n/i,$resultdata);

	# This line is here to ensure no duplicate votes loaded.

	$tally[$poll_num] = ();
	foreach $line (@resultarr) {
		if (substr($line,0,1) ne "#") {
			($ip, $value) = split(/=/i,$line);
			$results[$poll_num]{$ip} = $value;
			$tally[$poll_num][$value]++;
		}
	}

###
# This little section just trades blank result values for 0's for easy reading.
###
	for ($num = 0; $num <= $#{$options[$poll_num]};$num++) {
		$tally[$poll_num][$num] = int($tally[$poll_num][$num]);	
	}
# end load_poll sub.
}
