<?
#inc.data.php

function sanitize($input) {
	# This function is intended to sanitize user input from forms.
	# Mainly, we want to make sure people don't inject SQL requests in
	# through the forms to h4x0r us. We also want to remove any chars
	# that we might use a delimiters for fields (commas, equals, etc)

	# The bad_chars are separated by |. most characters need to be escaped
	# because ereg_replace will consider them regular expression commands.

	# Remove escaped-apostrophes and normal apostrophes.
	# Remove backslashes too. And equals and parentheses.
	$bad_chars = ",|=|\\\'|\'|\)|\(|\\\\";

	$input = preg_replace("/(($bad_chars))/","",$input);
	return $input;
#End function sanitize
}

function myescape($input) {
	# This function is intended to escape user input in situations where
	# using sanitize() would be undesireable, such as for passwords. 
	# Basically, we just want to strip apostrophes and backslashes.

	$bad_chars = "\'|\\\\";
	$input = preg_replace("/(($bad_chars))/","",$input);

	return $input;
#End function myescape
} 

function array_rdiff_assoc($array1,$array2) {
	# Be careful - when there is a recursive difference, the values 
	#  from the FIRST array are kept. If an element exists in the
	#  second array but not the first, it is added to the return array.
	foreach ($array1 as $key => $value) {
		if (!isset($array2[$key])) {
			$array_out[$key] = $value;
		} else {
			$tmp = array_diff_assoc($array1[$key],$array2[$key]);
			if (count($tmp) > 0) {
				$array_out[$key] = $value;
			}
		}
	}
	foreach ($array2 as $key => $value) {
		if (!isset($array1[$key])) {
			$array_out[$key] = $value;
		} 
	}
	return $array_out;
} #End function array_rdiff_assoc

function array_xdiff_assoc($array1,$array2) {
	# Any elements missing from the first array are left missing in the return array.
	foreach ($array1 as $key => $value) {
		if (!isset($array2[$key])) {
			$array_out[$key] = $value;
		} else {
			$tmp = array_diff_assoc($array1[$key],$array2[$key]);
			if (count($tmp) > 0) {
				$array_out[$key] = $value;
			}
		}
	}
	return $array_out;
} #End function array_xdiff_assoc

function add_task($account,$assignedby,$date,$service,$command) {
	$inc_dbh = db_connect();
	$db_query = "INSERT INTO task_queue (account,assignedby,date,service,command)
	VALUES ('$account','$assignedby','$date','$service','$command')";
	$sth = pg_query($inc_dbh, $db_query);
	if (!$sth) {
		echo "DB Query was: $db_query<br>\n";
		echo "Error: ".pg_last_error($inc_dbh)."<br>\n";
		return 0;
	}
	return 1;
} #End function add_task

function delete_user($account,$assignedby) {
	#Return 0 on success, 1 on failure.
	if (!isset($account) || !isset($assignedby)) {
		return 1;
	}
	$inc_dbh = db_connect();
	$date = time();
	if (add_task($account,$assignedby,$date,"delete","delete=1")) {
		return 0;
	} else {
		return 1;
	}
}

# Email functions

function is_valid_email_addr($email,$user) {
#####
### $user is optional. It is used to exclude the user's reference email from
###  the DB query that searches for duplicates. If no $user is present, no
###  accounts in the users db is overlooked (good for new user signups)
###
### Return array(1,whatever) on success, array(0,reason) on failure.
#
	$inc_dbh = db_connect();

# Start a formatting check of the email address.
	for ($n = 0; $n < strlen($email);$n++) {
		$char = substr($email,$n,1);
		if (preg_match("/[a-z]/",$char)) {
		} elseif (preg_match("/[A-Z]/",$char)) {
		} elseif (preg_match("/[0-9]/",$char)) {
		} elseif (preg_match("/@/",$char)) {
			$got_at = 1;
			$at_pos = $n;
		} elseif (preg_match("/\./",$char)) {
			# periods are permitted characters, most of the time.
			if ($n == 0) {
				# Periods may not start an email addr.
				$errmsg = "Email addresses may not start with periods";
				return array(0,$errmsg);
			}
			if ($got_at) {
				if ($n == $at_pos + 1) {
					# periods may not come immediately
					# after an @
					$errmsg = "Email domains may not start with a period";
					return array(0,$errmsg);
				}
				$got_endperiod = 1;
			}
			if (substr($email,$n + 1,1) == "@") {
				# periods must be followed by an alphanumeric
				# character before reaching an @.
				$errmsg = "Email addresses may not have periods followed immediately by @";
				return array(0,$errmsg);
			}

		} elseif (preg_match("/-/",$char)) {
			# dashes are permitted characters.
		} elseif (preg_match("/_/",$char)) {
			# underscores are permitted characters.
		} else {
			# $char must be an invalid character.
			$errmsg = "Your email address has an unrecognized character ' $char '";
			return array(0,$errmsg);
		}

	#End for each character of the addr
	}
	#Now, ensure that there was an @ and a . after the @
	if (!$got_endperiod) {
		$errmsg = "Your email address has an invalid domain name.";
		return array(0,$errmsg);
	}

	### Format check complete. 
	### Now see if the address is on the banlist.

	if (is_banned("reference_email",$email)) {
		$errmsg = "Your email address is banned from VectorStar.";
		return array(0,$errmsg);
	}

	## Now see if the domain is on the banlist.
	$email_domain = substr($email,strpos($email,"@") + 1);
	if (is_banned("mail_domain",$email_domain)) {
		$errmsg = "That email domain is not allowed on our system.";
		return array(0,$errmsg);
	}

	$db_query = "SELECT reference_email FROM users WHERE reference_email = '$email'";
	if (strlen($user) > 0) {
		$db_query .= " AND username NOT LIKE '$user'"; 
	}
	$sth = pg_query($inc_dbh,$db_query);
	$numrows = pg_numrows($sth);
	if ($numrows > 0) {
		$errmsg = "That email address is already in use.";
		return array(0,$errmsg);
	}

	$db_query = "SELECT reference_email FROM signups_approved WHERE reference_email = '$email'";
	if (strlen($user) > 0) {
		$db_query .= " AND username NOT LIKE '$user'"; 
	}
	$sth = pg_query($inc_dbh,$db_query);
	$numrows = pg_numrows($sth);
	if ($numrows > 0) {
		$errmsg = "You have already signed up for an account (and it was approved)! Please wait for your email confirmation to arrive.";
		return array(0,$errmsg);
	}

	$db_query = "SELECT reference_email FROM signups WHERE reference_email = '$email'";
	if (strlen($user) > 0) {
		$db_query .= " AND username NOT LIKE '$user'"; 
	}
	$sth = pg_query($inc_dbh,$db_query);
	$numrows = pg_numrows($sth);
	if ($numrows > 0) {
		$errmsg = "You have already signed up for an account! Please wait for your first account request to be approved.";
		return array(0,$errmsg);
	}

	return array(1,"Address is valid");

#End function is_valid_email_addr
}

# user functions

function user_exists($username) {
	# Return 1 on success, 0 on failure.
	$inc_dbh = db_connect();
	$db_query = "SELECT username FROM users WHERE username = '$username'";
	$sth = pg_query($inc_dbh,$db_query);
	$numrows = pg_numrows($sth);
	return $numrows;
} # End function user_exists

function signup_exists($username) {
	# Return 1 on success, 0 on failure.
	$inc_dbh = db_connect();
	$db_query = "SELECT username FROM signups WHERE username = '$username'";
	$sth = pg_query($inc_dbh,$db_query);
	$numrows = pg_numrows($sth);
	if ($numrows > 0) {
		return $numrows;
	} else {
		$db_query = "SELECT username FROM signups_approved WHERE username = '$username'";
		$sth = pg_query($inc_dbh,$db_query);
		$numrows = pg_numrows($sth);
		return $numrows;
	}
} # End function signup_exists

function is_banned($field,$value) {
	# Return 1 on success, 0 on failure.
	$inc_dbh = db_connect();
	$db_query = "SELECT $field FROM users_banned";
	$sth = pg_query($inc_dbh,$db_query);
	$numrows = pg_numrows($sth);
	for ($n = 0; $n < $numrows;$n++) {
		$result = pg_fetch_row($sth,$n);
		# If the value being tested contains any of the banned
		#  fields as a substring, return a match.
		if (!is_bool(stristr($value,$result[0]))) {
			return 1;
		}
	}
	return 0;
} # End function is_banned.

function is_valid_username($username) {
	# Return 1 on success, 0 on failure.
	for ($n = 0; $n < strlen($username);$n++) {
		$char = substr($username,$n,1);
		if (!preg_match("/[a-z]|[0-9]/",$char)) {
			return 0;
		}
	}
	if (is_banned("username",$username)) {
		return 0;
	}
	return 1;
} #End ofunction is_valid_username

# web functions

function is_valid_web_domain($domain,$user) {
	if (isset($user)) {
		$inc_dbh = db_connect();
		$db_query = "SELECT config FROM features WHERE username = '$user' AND feature = 'web_domain' AND config = '$domain'";
		$sth = pg_query($inc_dbh,$db_query);
		$numrows = pg_numrows($sth);
		if ($numrows == 1) {
			# This person already has this web domain configured - leave them be.
			return 1;
		}
	}
	# Check to see if anyone else already has this domain name.
	$db_query = "SELECT config FROM features WHERE feature = 'web_domain' AND config = '$domain'";
	$sth = pg_query($inc_dbh,$db_query);
	$numrows = pg_numrows($sth);
	if ($numrows == 1) {
		# Someone already has this web domain configured - Fail.
		return 0;
	}
	# Return 1 on success, 0 on failure.
	if (strlen($domain) == 0) {
		return 0;
	}
	# First things first, do a character check.
	for ($n = 0;$n < strlen($domain);$n++) {
		$char = substr($domain,$n,1);
		if (!preg_match("/[a-z]|[0-9]|-|\./",$char)) {
			return 0;
		}
	} #End for

	if (preg_match("/\.vectorstar.net$/",$domain)) {
		$just_subdomain = ereg_replace(".vectorstar.net","",$domain);
		if (preg_match("/\./",$just_subdomain)) {
			return 0;
		}
		# Do the great pingtest.
		$pingtest = `ping -c 2 $domain 2>&1`;
		if (stristr($pingtest,"ping: unknown host $domain")) {
			return 1;
		} else {
			return 0;
		}
	} else {
		# This must be a full domain.
		# These are no longer permitted (site hijacking problem)
		return 0;
	}
} #End function is_valid_web_domain

?>
